import SockJS from 'sockjs-client';
import Stomp from 'stompjs';

function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
}

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

var CONNECT_STATE;

(function (CONNECT_STATE) {
  CONNECT_STATE[CONNECT_STATE["DISCONNECTED"] = 0] = "DISCONNECTED";
  CONNECT_STATE[CONNECT_STATE["CONNECTING"] = 1] = "CONNECTING";
  CONNECT_STATE[CONNECT_STATE["CONNECTED"] = 2] = "CONNECTED";
})(CONNECT_STATE || (CONNECT_STATE = {}));

var BREAK_REASON;

(function (BREAK_REASON) {
  BREAK_REASON[BREAK_REASON["INVALID_TOKEN"] = 0] = "INVALID_TOKEN";
  BREAK_REASON[BREAK_REASON["MANUAL_DISCONNECT"] = 1] = "MANUAL_DISCONNECT";
})(BREAK_REASON || (BREAK_REASON = {}));

var SockClient =
/*#__PURE__*/
function () {
  function SockClient(opts) {
    _classCallCheck(this, SockClient);

    this._opts = opts;
    this._state = CONNECT_STATE.DISCONNECTED;
    this.validate();
    this._stateChangeCallbacks = [];
    this._broadcastCallbacks = {};
    this._singleSubscribeCallback = [];
    this._subscriptions = [];
    this._breakReason = null;
  }

  _createClass(SockClient, [{
    key: "validate",
    value: function validate() {
      var _this = this;
      ['base', 'projectId', 'token'].forEach(function (key) {
        if (!_this._opts[key]) {
          throw new Error("".concat(key, " is missed in passed opts"));
        }
      });
    }
  }, {
    key: "connect",
    value: function connect() {
      if (this._socket || this._stompClient) {
        throw new Error('You cannot call connect multiple times');
      }

      this.connectToWebsocket();
    }
  }, {
    key: "disconnect",
    value: function disconnect() {
      try {
        this.changeState(CONNECT_STATE.DISCONNECTED);
        this.setBreakReason(BREAK_REASON.MANUAL_DISCONNECT, false);
        this._stateChangeCallbacks = [];
        this._broadcastCallbacks = {};
        this._singleSubscribeCallback = [];
        this._subscriptions = [];
        var socket = this._socket;
        this._socket = null;
        this._stompClient = null;
        socket.close();
      } catch (error) {
        console.warn('Closing ', error);
      }
    }
  }, {
    key: "onStateChange",
    value: function onStateChange(cb) {
      if (!cb) {
        return;
      }

      if (this._stateChangeCallbacks.every(function (c) {
        return c !== cb;
      })) {
        this._stateChangeCallbacks.push(cb);
      }
    }
  }, {
    key: "changeState",
    value: function changeState(state) {
      var _this2 = this;

      this._state = state;

      this._stateChangeCallbacks.forEach(function (cb) {
        cb(_this2._state);
      });

      if (state === CONNECT_STATE.CONNECTED) {
        this.startSubscribe();
      }

      if (state === CONNECT_STATE.DISCONNECTED) {
        this.endSubscribe();
      }
    }
  }, {
    key: "startSubscribe",
    value: function startSubscribe() {
      var _this3 = this;

      var topics = Object.keys(this._broadcastCallbacks);
      topics.forEach(function (topic) {
        var subscription = _this3._stompClient.subscribe("/topic/".concat(_this3._opts.projectId, "/").concat(topic), function (arg) {
          _this3._broadcastCallbacks[topic].forEach(function (cb) {
            cb(arg);
          });
        });

        _this3._subscriptions.push(subscription);
      });

      var subscription = this._stompClient.subscribe("/user/queue/".concat(this._opts.projectId, "/"), function (arg) {
        _this3._singleSubscribeCallback.forEach(function (cb) {
          cb(arg);
        });
      });

      this._subscriptions.push(subscription);
    }
  }, {
    key: "endSubscribe",
    value: function endSubscribe() {
      this._subscriptions.forEach(function (sub) {
        sub.unsubscribe();
      });
    }
  }, {
    key: "subscribeBroadcast",
    value: function subscribeBroadcast(topic, cb) {
      if (!topic || !cb) {
        throw new Error('topic, cb cannot be undefined/null');
      }

      if (!this._broadcastCallbacks[topic]) {
        this._broadcastCallbacks[topic] = [];
      }

      if (this._broadcastCallbacks[topic].every(function (c) {
        return c !== cb;
      })) {
        this._broadcastCallbacks[topic].push(cb);
      }
    }
  }, {
    key: "subscribeForCurrentUser",
    value: function subscribeForCurrentUser(cb) {
      if (this._singleSubscribeCallback.every(function (c) {
        return c !== cb;
      })) {
        this._singleSubscribeCallback.push(cb);
      }
    }
  }, {
    key: "connectToWebsocket",
    value: function connectToWebsocket() {
      var _this4 = this;

      var base = this._opts.base;
      this._socket = SockJS(base);
      this._stompClient = Stomp.over(this._socket);
      var _this$_opts = this._opts,
          token = _this$_opts.token,
          projectId = _this$_opts.projectId;
      this.changeState(CONNECT_STATE.CONNECTING);
      console.debug('Trying to connect to Websocket Server...');

      this._stompClient.connect({
        token: token,
        projectId: projectId
      }, function () {
        console.debug('Websocket connection established...');

        _this4.changeState(CONNECT_STATE.CONNECTED);
      }, function (error) {
        _this4.changeState(CONNECT_STATE.DISCONNECTED);

        if (_this4._breakReason === BREAK_REASON.INVALID_TOKEN) {
          // error callback will be called twice, that's why we record _breakReason with first call
          // we should just leave the callback if _breakReason from last call exist and match invalid_token
          return;
        }

        if (_this4._breakReason === BREAK_REASON.MANUAL_DISCONNECT) {
          // do not re-connect if it is disconnected manually
          return;
        } // quit here since token is checked as invalid by server
        // no need to re-connect with same arguments


        if (error && error['headers'] && error['headers']['message'] && error['headers']['message'].includes('Failed to send message')) {
          _this4.setBreakReason(BREAK_REASON.INVALID_TOKEN, true);

          return;
        }

        _this4.reConnectWebSocket();
      });
    }
  }, {
    key: "reConnectWebSocket",
    value: function reConnectWebSocket() {
      var _this5 = this;

      var reconnect = this._opts.reconnect;

      if (!reconnect) {
        return;
      }

      var retryTimeout = reconnect.timeout || 30 * 1000;
      console.debug("Re-connecting websocket after ".concat(retryTimeout / 1000, " secs...")); // Call the websocket connect method

      setTimeout(function () {
        _this5.connectToWebsocket();
      }, retryTimeout);
    }
  }, {
    key: "setBreakReason",
    value: function setBreakReason(reason, needClear) {
      var _this6 = this;

      this._breakReason = reason;

      if (needClear) {
        // clear _breakReason in 2s
        setTimeout(function () {
          _this6._breakReason = null;
        }, 2000);
      }
    }
  }]);

  return SockClient;
}();

export { BREAK_REASON, CONNECT_STATE, SockClient };
